// Kenneth Bogert (kbogert   at yahoo dot com)
//
// A very simple D class wrapper around libcurl's easy interface.  There is much room for improvement.

import curldef;
import std.string;
import std.uri;
debug (CURLDUMP) { import std.stdio; }

struct ProxyData {
	char [] url;
	char [] userName;
	char [] password;
	curl_proxytype type;
	long authType;
}

struct FormField {
	char [] name;
	char [] value;
}

class LibCurlException: Exception
{
	this(char[] msg)
	{
		super(msg);
	}
}

class LibCurl {
	private:
		CURL * mCurl;
		char [] url;
		char [CURL_ERROR_SIZE] lastErrorMessage;
		ProxyData mProxyData;
	
		
		extern (C) {

			static int stringWriter(char * newData, size_t size, size_t nmemb, inout char [] data) {
		
				data ~= newData[0 .. (size * nmemb)];

				return size * nmemb;
			}

			debug (CURLDUMP) {
				static int debugHeaderWriter(CURL * myCurl, curl_infotype type, char * myData, size_t size, void * not) {
		
					writefln("%s", myData[0 .. size]);

					return 0;
				}
			}
		}

		void init() {
			mCurl = curl_easy_init();
			if (mCurl is null)
				throw new LibCurlException("Exception Initializing CURL: " ~ lastErrorMessage);
		
			setOption( CURLOPT_ERRORBUFFER, &lastErrorMessage);		
			setOption( CURLOPT_COOKIEFILE, "");
			setOption( CURLOPT_FOLLOWLOCATION, 1);
			debug (CURLDUMP) {
				setOption( CURLOPT_DEBUGFUNCTION, &debugHeaderWriter);		
				setOption( CURLOPT_HEADER, 1);
				setOption( CURLOPT_VERBOSE, 1);
			}
		}
	

	public:

	// TODO add event delegates here for progress, headers, and ssl context
	
	this() {
		init();
	}
	
	~this() {
		curl_easy_cleanup(mCurl);
	}
	
	void URL(char [] newURL) {
		url = newURL;
		setOption( CURLOPT_URL, url);
	}
	
	char [] URL () {
		return url;
	}
	
	void proxy(ProxyData pData) {
		mProxyData = pData;
		setOption( CURLOPT_PROXY, pData.url);
		if (pData.userName != null) {
			setOption( CURLOPT_PROXYUSERPWD, std.uri.encode(pData.userName) ~ ":" ~ std.uri.encode(pData.password));
		}
		setOption( CURLOPT_PROXYTYPE, pData.type);
		if (pData.authType != 0)
			setOption( CURLOPT_PROXYAUTH, pData.authType);		
		else 
			setOption( CURLOPT_PROXYAUTH, CURLAUTH_ANY);
	}
	
	ProxyData proxy() {
		return mProxyData;
	}
	
	void setOption(CURLoption opt, char [] optData) {		
		CURLcode code;
		code = curl_easy_setopt(mCurl, opt, std.string.toStringz(optData));
		if (code != CURLE_OK) {
			throw new LibCurlException("Exception Setting Option: " ~ lastErrorMessage);
		}		
	}
	
	void setOption(CURLoption opt, int optData) {		
		CURLcode code;
		code = curl_easy_setopt(mCurl, opt, optData);
		if (code != CURLE_OK) {
			throw new LibCurlException("Exception Setting Option: " ~ lastErrorMessage);
		}		
	}

	void setOption(CURLoption opt, bool optData) {		
		CURLcode code;
		code = curl_easy_setopt(mCurl, opt, optData);
		if (code != CURLE_OK) {
			throw new LibCurlException("Exception Setting Option: " ~ lastErrorMessage);
		}		
	}

	void setOption(CURLoption opt, void * optData) {		
		CURLcode code;
		code = curl_easy_setopt(mCurl, opt, optData);
		if (code != CURLE_OK) {
			throw new LibCurlException("Exception Setting Option: " ~ lastErrorMessage);
		}		
	}
	
	char [] downloadToString() {
		setOption( CURLOPT_WRITEFUNCTION, &stringWriter);
		char [] data = "";
		setOption( CURLOPT_WRITEDATA, &data);

		CURLcode code = curl_easy_perform(mCurl);
		if (code != CURLE_OK) {
			throw new LibCurlException("Failed to Download: " ~ url ~ " - " ~ lastErrorMessage);
		}

		setOption( CURLOPT_HTTPGET, 1);

		return data;

	}
	
	void postForm(FormField [] data) {
		setOption( CURLOPT_POST, 1);
		char [] fieldBuffer;
		foreach (FormField curForm; data) {
			fieldBuffer ~= std.uri.encode(curForm.name) ~ "=" ~ std.uri.encode(curForm.value) ~ "&";
		}
		if (fieldBuffer.length > 0){
			fieldBuffer = fieldBuffer[0 .. fieldBuffer.length - 1];
			setOption( CURLOPT_POSTFIELDS, fieldBuffer);
		}
		
	}
	
	void restart() {
		curl_easy_cleanup(mCurl);
		init();
	}
	
	unittest {
		
		LibCurl myCurl = new LibCurl();
		myCurl.URL = "http://www.google.com";

		char [] results = myCurl.downloadToString();
		
		assert( results.length > 0);
	
		myCurl.URL = "https://login.yahoo.com";
		myCurl.setOption( CURLOPT_SSL_VERIFYPEER, false);
	
		results = myCurl.downloadToString();
	
		assert( results.length > 0);
	
		myCurl.restart();
	
		myCurl.URL = "http://scphillips.com/cgi-bin/convert.cgi";
		FormField [4] calculator;
		calculator[0].name = "form";
		calculator[0].value = "1";
		calculator[1].name = "from";
		calculator[1].value = "acre";
		calculator[2].name = "to";
		calculator[2].value = "mile^2";
		calculator[3].name = "amount";
		calculator[3].value = "15";
		myCurl.postForm(calculator);
		results = myCurl.downloadToString();
	
		assert( results.length > 0);
		assert( find(results, "0.0234375") > 0);
	}
	
}